import { attachShadow, h, Host, proxyCustomElement } from '@stencil/core/internal/client';
export { setAssetPath } from '@stencil/core/internal/client';

const TAG_I18N_TRANSLATIONS = '__i18njs-data';
var i18nEvents;
(function (i18nEvents) {
  i18nEvents["updateTranslations"] = "i18n-update-translations";
})(i18nEvents || (i18nEvents = {}));
function raiseEvent(eventName) {
  document.dispatchEvent(new Event(eventName));
}
/**
 * Devuelve la tag que se usará para el localstorage, compuesta por estos
 * elementos, separados por `_`:
 * - TAG estándar `TAG_I18N_TRANSLATIONS`
 * - El idioma de las traducciones
 * - La versión, si no es `undefined` siempre se pone en el último tramo.
 * @param lang idioma de las traducciones
 * @param version versión de las traducciones
 */
function getTag(lang, version) {
  return `${TAG_I18N_TRANSLATIONS}_${lang}_${version || ''}`;
}
function getStorageT(lang, version) {
  if (version >= 0) {
    const translations = JSON.parse(localStorage.getItem(getTag(lang, version)));
    if (Object.keys(translations).length > 0) {
      return translations;
    }
  }
  return {};
}
function setStorageT(lang, version, translations) {
  localStorage.setItem(getTag(lang, version), JSON.stringify(translations));
}
function removeStorageT(lang) {
  Object.keys(localStorage)
    .filter(key => key.includes(getTag(lang)))
    .map(key => localStorage.removeItem(key));
}
function clearLocalStorage() {
  Object.keys(localStorage)
    .filter(key => key.startsWith(TAG_I18N_TRANSLATIONS))
    .map(key => localStorage.removeItem(key));
}
/**
 * Devuelve la versión, solamente habrá una versión en localstorage, ya que
 * la anterior se elimina antes de guardar la nueva.
 * @param lang El idioma para el que se quieren las traducciones
 */
function getStorageVersion(lang, defaultVersion = '-1') {
  const tag = Object.keys(localStorage).find((tag) => tag.includes(getTag(lang)));
  const version = tag ? tag.split('_').pop() : defaultVersion;
  return parseFloat(version);
}

class Language {
  constructor(defaultLanguage = 'en', isLocalValuesAllowed = false, userLanguage = null) {
    this.defaultLanguage = 'en';
    this.isLocalValuesAllowed = false;
    this.userLanguage = null;
    this.defaultLanguage = defaultLanguage;
    this.language = defaultLanguage;
    this.isLocalValuesAllowed = isLocalValuesAllowed;
    this.userLanguage = userLanguage;
  }
  setValues(defaultLanguage) {
    this.defaultLanguage = defaultLanguage;
  }
  setLocalValuesAllowed(isLocalValuesAllowed = false) {
    this.isLocalValuesAllowed = isLocalValuesAllowed;
    this.manageCookie(this.language);
  }
  /**
  * Comprueba si el valor introducido se encuentra en availableLangs (Lenguajes disponibles).
  * @param lang Dialecto (p.ej) `ue`, `en-ue`
  * @returns EL lang introducido o si no lo encuentra el defaultLanguage.
  */
  getFromAvailableDialects(availableLangs, lang) {
    const language = lang.indexOf('-') < 0 ? lang : lang.split('-')[0];
    return availableLangs.find(l => l.indexOf(language) === 0) || this.defaultLanguage;
  }
  getFromAvailableLangs(availableLangs, lang) {
    if (availableLangs.indexOf(lang.toLowerCase()) >= 0) {
      return lang.toLowerCase();
    }
    return this.getFromAvailableDialects(availableLangs, lang);
  }
  initLanguage(availableLangs, resetCookie = false) {
    let lang;
    if (this.isLocalValuesAllowed) {
      const [, cookieLang] = document.cookie.trim().match(/lang=([^;]*)/) || [];
      if (cookieLang) {
        const [, value] = cookieLang;
        lang = this.getFromAvailableLangs(availableLangs, value);
      }
    }
    if (!lang || resetCookie) {
      lang = this.userLanguage ||
        window.navigator.userLanguage ||
        window.navigator.language ||
        this.defaultLanguage;
      lang = this.getFromAvailableLangs(availableLangs, lang);
      this.manageCookie(lang);
    }
    if (lang !== this.language) {
      this.language = lang;
      raiseEvent(i18nEvents.updateTranslations);
    }
  }
  getLanguage(availableLangs, resetCookie = false) {
    if (!this.language) {
      this.initLanguage(availableLangs, resetCookie);
    }
    return this.language;
  }
  setLanguage(availableLangs, lang) {
    if (this.language !== lang) {
      this.language = this.getFromAvailableLangs(availableLangs, lang);
      this.userLanguage = this.language;
      this.manageCookie(this.language);
      raiseEvent(i18nEvents.updateTranslations);
    }
  }
  manageCookie(lang) {
    if (this.isLocalValuesAllowed) {
      const exdate = new Date();
      exdate.setDate(exdate.getDate() + 20 * 365);
      document.cookie = `lang=${lang};expires=${exdate.toUTCString()};path=/`;
    }
    else {
      document.cookie = 'lang=;expires=Thu, 01 Jan 1970 00:00:01 GMT;path=/;';
    }
  }
}

class TranslationsDownloader {
  constructor(apiUrl, appId, appSecret, missingTag = 'app', tags = []) {
    this.tags = [];
    this.tInterval = -1;
    this.missingsInterval = {};
    this.translationsCache = {};
    this.availableLangsCache = [];
    this.notFoundCache = [];
    this.missingTag = 'app';
    this.requestOptions = {
      'Content-Type': 'application/json',
      'Access-Control-Allow-Origin': location.origin,
      'x-translate-i18n': 'js',
    };
    this.apiUrl = apiUrl;
    this.appId = appId;
    this.appSecret = appSecret;
    this.missingTag = missingTag;
    this.tags = tags;
  }
  setValues(apiUrl, appId, appSecret) {
    this.apiUrl = apiUrl;
    this.appId = appId;
    this.appSecret = appSecret;
  }
  cacheClear() {
    this.translationsCache = {};
    this.availableLangsCache = [];
  }
  /**
  * Obtiene las traducciones para un idioma y los idiomas disponibles.
  * Primero mira en la cache y sino las obtiene de (p.ej. para es): `assets/translations/all-es.json`
  * @param lang Idiioma de las traducciones
  * isForceRefresh
  * @returns Un objeto json con las traducciones disponibles para ese idioma
  */
  async getTranslationsConfig(defaultAvailableLangs, lang, isLocalValuesAllowed) {
    // - Cancelar el intervalo anterior de refresco
    // const isIdle = this.tInterval < 0;
    clearInterval(this.tInterval);
    // - Miramos si están en cache
    if (this.translationsCache[lang]) {
      return {
        availableLangs: this.availableLangsCache,
        translations: this.translationsCache[lang],
      };
    }
    // - Obtememos las traducciones locales
    const ts = await this.getLocal(lang, isLocalValuesAllowed);
    const remoteAvailableLangs = defaultAvailableLangs;
    this.translationsCache[lang] = ts;
    this.availableLangsCache = defaultAvailableLangs;
    // - Miramos que no se haya iniciado ya anteriormente el proceso de descarga en background
    if (this.tInterval < 0) {
      // - Iniciamos el descargador de las remote en background
      this.tIntervalCount = 0;
      this.tInterval = setInterval(async () => {
        try {
          const remoteAvailableLangs = await this.getRemoteAvailableLangs(defaultAvailableLangs);
          const ts = await this.getRemote(lang, isLocalValuesAllowed);
          this.translationsCache[lang] = ts;
          this.availableLangsCache = remoteAvailableLangs;
          clearInterval(this.tInterval);
          this.tInterval = -1;
          raiseEvent(i18nEvents.updateTranslations);
        }
        catch (err) {
          console.error(err);
        }
        if (this.tIntervalCount >= 6) {
          // Timeout
          clearInterval(this.tInterval);
          this.tInterval = -1;
        }
        else {
          this.tIntervalCount += 1;
        }
      }, 10000);
    }
    return {
      availableLangs: remoteAvailableLangs,
      translations: ts
    };
  }
  async getRemoteAvailableLangs(defaultAvailable) {
    if (this.apiUrl) {
      const authQs = await this.createAuthQs(this.appId, this.appSecret);
      const availableUrl = `${this.apiUrl}/lang?${authQs}`;
      const resp = await fetch(availableUrl, {
        method: 'GET',
        headers: this.requestOptions,
        credentials: 'include',
      });
      if (resp.ok) {
        // No hay respuesta del servidor o ya tenemos la ultima locVersion en localstorage
        return (await resp.json()) || defaultAvailable;
      }
    }
    else {
      throw new Error('API URL was not configured');
    }
    return defaultAvailable;
  }
  async getRemote(lang, isLocalValuesAllowed) {
    if (this.apiUrl) {
      const authQs = await this.createAuthQs(this.appId, this.appSecret);
      const storageVersion = getStorageVersion(lang);
      const vUrl = `${this.apiUrl}/lang/loc-version?lang=${lang}&${authQs}`;
      // gestion de la version
      const vResponse = await fetch(vUrl, {
        method: 'GET',
        headers: this.requestOptions,
        credentials: 'include',
      });
      const locVersionStr = (await vResponse.text()) || '-1';
      const locVersion = parseFloat(locVersionStr);
      if (!vResponse.ok || locVersion < 0 || locVersion === storageVersion) {
        // No hay respuesta del servidor o ya tenemos la ultima locVersion en localstorage
        return this.getLocal(lang, isLocalValuesAllowed);
      }
      // hay una nueva version en el servidor => la descargamos
      let allUrl = `${this.apiUrl}/locale/all.json?lang=${lang}&${authQs}`;
      if (this.tags && this.tags.length > 0) {
        allUrl += `&tags=${this.tags.join(',')}`;
      }
      const locResponse = await fetch(allUrl, {
        method: 'GET',
        headers: this.requestOptions,
        credentials: 'include',
      });
      if (locResponse.ok) {
        // Elimina la `${TAG_I18N_TRANSLATIONS}_${lang}_${version}` para el idioma seleccionado
        removeStorageT(lang);
        const translations = this.process(await locResponse.json());
        setStorageT(lang, locVersion, translations);
        return translations;
      }
      throw new Error(`${locResponse.statusText}: ${await locResponse.text()}`);
    }
    else {
      throw new Error('API URL was not configured');
    }
  }
  async getLocal(lang, isLocalValuesAllowed) {
    // Check if available in localstorage and otherwise failback to assets
    const storageVersion = getStorageVersion(lang);
    let storageTranslations = getStorageT(lang, storageVersion);
    const assetsVersion = await fetch('assets/translations/versions.json');
    let version = 0;
    if (assetsVersion.ok) {
      const versionJson = (await assetsVersion.json()) || {};
      const versionStr = versionJson[`version_${lang}`] || '0';
      version = parseFloat(versionStr);
    }
    if (storageVersion < 0 || Object.keys(storageTranslations).length === 0 || storageVersion < version) {
      // En caso que la versión del localstorage no exista, que las traducciones del localstorage estén vacías
      // o que la versión del localstorage sea menor que la versión en assets/translations/versions.json
      // hay que leer los datos en assets/translations/all.json y guardarlos en locastorage
      const assetsTranslations = await fetch(`assets/translations/all${lang ? '-' : ''}${lang}.json`);
      if (assetsTranslations.status === 404 && lang !== '') {
        // No existen valores para este idioma, cogemos los valores por defecto
        return this.getLocal('', isLocalValuesAllowed);
      }
      if (!assetsTranslations.ok) {
        return {};
      }
      // Procesamos las traducciones y las guardamos en localstorage
      const translations = this.process(await assetsTranslations.json());
      setStorageT(lang, version, translations);
      storageTranslations = translations;
    }
    return storageTranslations;
  }
  async createAuthQs(appId, appSecret) {
    if (appId && appSecret) {
      const nonce = new Date().getTime();
      const token = await this.generateToken(`${appSecret}-${nonce}`);
      return `appId=${encodeURIComponent(appId)}&token=${encodeURIComponent(token)}&timestamp=${nonce}`;
    }
    return '';
  }
  async generateToken(password) {
    try {
      const hashBytes = 32;
      const saltBytes = 40;
      const iterations = 100;
      const digest = 'SHA-512';
      const pwBuf = this.str2ab(password);
      // First, create a PBKDF2 "key" containing the password
      const baseKey = await window.crypto.subtle.importKey('raw', pwBuf, 'PBKDF2', false, ['deriveBits', 'deriveKey']);
      // Generate random salt
      const salt = new Uint8Array(saltBytes);
      window.crypto.getRandomValues(salt);
      // Derive a key from the password and random salt
      const params = { iterations, salt, name: 'PBKDF2', hash: digest };
      const hash = await crypto.subtle.deriveBits(params, baseKey, hashBytes * 8);
      const hashUint8 = new Uint8Array(hash);
      // Create combined token
      const combined = new ArrayBuffer(salt.length + hashUint8.length + 8);
      const dataview = new DataView(combined);
      // Write data into combined buffer
      // Server is using Big Endian
      dataview.setUint32(0, salt.length, false);
      dataview.setUint32(4, params.iterations, false);
      for (let i = 0; i < salt.length; i += 1) {
        dataview.setUint8(i + 8, salt[i]);
      }
      for (let i = 0; i < hashUint8.length; i += 1) {
        dataview.setUint8(i + salt.length + 8, hashUint8[i]);
      }
      // Return as base64 string
      const combinedS = this.ab2str(combined);
      return btoa(combinedS);
    }
    catch (err) {
      console.error(`Key derivation failed: ${err.message}`);
    }
  }
  ab2str(buffer) {
    const byteArray = new Uint8Array(buffer);
    let byteString = '';
    for (let i = 0; i < byteArray.byteLength; i += 1) {
      byteString += String.fromCodePoint(byteArray[i]);
    }
    return byteString;
  }
  str2ab(byteString) {
    const byteArray = new Uint8Array(byteString.length);
    for (let i = 0; i < byteString.length; i += 1) {
      byteArray[i] = byteString.codePointAt(i);
    }
    return byteArray;
  }
  /**
  * Fills notFound array with empty translation strings.
  * @param object translations json.
  */
  process(object) {
    return Object.keys(object).reduce((newObject, key) => {
      if (typeof object[key] === 'object') {
        newObject[key] = this.process(object[key]);
      }
      else if ((typeof object[key] === 'string') && (object[key] === '')) {
        this.notFoundCache.push(key);
      }
      else {
        newObject[key] = object[key];
      }
      return newObject;
    }, {});
  }
  handleMissing(translationID, lang) {
    if (this.notFoundCache.indexOf(translationID) < 0) {
      clearInterval(this.missingsInterval[translationID]);
      this.notFoundCache.push(translationID);
      if (this.apiUrl) {
        // Report if not yet reported or retry after 5 minutes
        this.missingsInterval[`${translationID}-count`] = 0;
        this.missingsInterval[translationID] = setInterval(async () => {
          try {
            const authQs = await this.createAuthQs(this.appId, this.appSecret);
            const data = {
              lang,
              tag: this.missingTag,
              text: translationID,
              extra: window.location.href,
            };
            await fetch(`${this.apiUrl}/locale/missing?${authQs}`, {
              method: 'POST',
              headers: this.requestOptions,
              credentials: 'include',
              body: JSON.stringify(data),
            });
            clearInterval(this.missingsInterval[translationID]);
          }
          catch (error) {
            console.error(error);
          }
          if (this.missingsInterval[`${translationID}-count`] >= 6) {
            clearInterval(this.missingsInterval[translationID]);
          }
          else {
            this.missingsInterval[`${translationID}-count`] += 1;
          }
        }, 10000);
      }
    }
    return translationID;
  }
}

/**
 * Esta clase gestiona todo el contenido de las traducciones
 */
class Translator {
  constructor() {
    this.isLocalValuesAllowed = false;
    this.availableLangs = ['en'];
  }
  static getInstance() {
    if (!Translator.instance) {
      throw new Error('Translator is not yet initialized, please call Translator.init() first');
    }
    return Translator.instance;
  }
  static init(options) {
    let { instance } = Translator;
    const { apiUrl, appId, appSecret, availableLangs = ['en'], // TODO: GET /i18n/langs
    defaultLanguage = 'en', isShowMissing = false, isLocalValuesAllowed = false, userLanguage = null, missingTag = 'app', tags = [], } = options;
    if (instance) {
      // Cambia los parametros de inicializacion
      instance.availableLangs = availableLangs;
      instance.isLocalValuesAllowed = isLocalValuesAllowed;
      instance.language.setValues(defaultLanguage);
      instance.tDonwloader.setValues(apiUrl, appId, appSecret);
      return instance;
    }
    instance = new Translator();
    instance.tDonwloader = new TranslationsDownloader(apiUrl, appId, appSecret, missingTag, tags);
    instance.availableLangs = availableLangs;
    instance.isLocalValuesAllowed = isLocalValuesAllowed;
    instance.language = new Language(defaultLanguage, isLocalValuesAllowed, userLanguage);
    instance.language.initLanguage(availableLangs);
    instance.isShowMissing = isShowMissing;
    Translator.instance = instance;
    return instance;
  }
  static setLocalValuesAllowed(isLocalValuesAllowed = false) {
    this.getInstance().setLocalValuesAllowed(isLocalValuesAllowed);
  }
  setLocalValuesAllowed(isLocalValuesAllowed = false) {
    this.language.setLocalValuesAllowed(isLocalValuesAllowed);
    this.isLocalValuesAllowed = isLocalValuesAllowed;
    if (!isLocalValuesAllowed) {
      clearLocalStorage();
    }
  }
  /**
  * Tries to translate a text, returns the text itself if no translation is found.
  * @param originalText text to translate
  * @param tData Interpolation parameters
  * @param lang  Translation language
  * @returns Translated text
  */
  static t(originalText, tData, lang) {
    return this.getInstance().t(originalText, tData, lang);
  }
  /**
   * Tries to translate a text, returns the text itself if no translation is found.
   * @param originalText text to translate
   * @param tData Interpolation parameters
   * @param lang  Translation language
   * @returns Translated text
   */
  // tslint:disable-next-line: function-name
  async t(originalText, tData, lang) {
    const selectedLanguage = lang || this.language.getLanguage(this.availableLangs);
    const { availableLangs, translations } = await this.tDonwloader.getTranslationsConfig(this.availableLangs, selectedLanguage, this.isLocalValuesAllowed);
    this.availableLangs = availableLangs;
    let translated = originalText;
    if (!translations[originalText]) {
      // Missing to server
      const missingText = await this.tDonwloader.handleMissing(originalText, selectedLanguage);
      if (translations[originalText] === '' && this.isShowMissing) {
        // La cadena ya se ha creado, pero todavía no se ha traducido
        console.warn(`Still missing translation for "${originalText}"`);
      }
      else {
        console.warn(`Missing translation for "${missingText}"`);
      }
      translated = originalText;
    }
    else {
      translated = translations[originalText];
    }
    return tData ? this.interpolate(translated, tData) : translated;
  }
  /**
  * *Función para uso interno*. Recorre las keys del JSON para comprobar si coinciden con el RegExp (p.ej) : `(%key%)`,
  * si coinciden sustituye la palabra por el valor encontrado en el JSON (p.ej) : `(%nombre%)` por `Toni`
  * @param text Texto a interpolar
  * @param data Json que contiene los paramatros del componente (p.ej) : `{ nombre: 'Toni', edad: 25 }`
  * @returns El texto interpolado
  */
  interpolate(text, data) {
    let interpolated = text;
    if (text.indexOf('(%') > -1) {
      Object.keys(data).forEach((tkey) => {
        const regex = new RegExp(`\\(%\\s*${tkey}\\s*%\\)`, 'g');
        interpolated = interpolated.replace(regex, data[tkey] || '');
      });
    }
    return interpolated;
  }
  /**
   * Changes the language of the application.
   * @param lang new language
   */
  static changeLanguage(lang) {
    this.getInstance().changeLanguage(lang);
  }
  /**
   * Changes the language of the application.
   * @param lang new language
   */
  changeLanguage(lang) {
    this.language.setLanguage(this.availableLangs, lang);
  }
  /**
   * Clears the translations cache and tries to download again the translations.
   */
  static cacheClear() {
    this.getInstance().cacheClear();
  }
  /**
   * Clears the translations cache and tries to download again the translations.
   */
  cacheClear() {
    this.tDonwloader.cacheClear();
  }
}

const i18nTCss = ":host{display:inline}";

const I18nT = class extends HTMLElement {
  constructor() {
    super();
    this.__registerHost();
    attachShadow(this);
    this.translator = Translator.getInstance();
    this.originalText = this.i18nTEL.innerHTML;
    this.getTranslation();
  }
  async getTranslation() {
    let tData = {};
    try {
      tData = JSON.parse(this.tData || '{}');
    }
    catch (e) { }
    this.i18nTEL.innerHTML = await this.translator.t(this.originalText, tData);
  }
  /**
   *
   */
  async handleTranslationsUpdate() {
    this.getTranslation();
  }
  /**
   *
   * @param data
   */
  async handleLanguageChange(data) {
    this.translator.changeLanguage(data.detail);
  }
  render() {
    return h(Host, null, h("slot", null));
  }
  get i18nTEL() { return this; }
  static get style() { return i18nTCss; }
};

const I18nT$1 = /*@__PURE__*/proxyCustomElement(I18nT, [1,"i18n-t",{"tData":[1,"t-data"]},[[4,"i18n-update-translations","handleTranslationsUpdate"],[4,"i18n-language-change","handleLanguageChange"]]]);
const defineCustomElements = (opts) => {
  if (typeof customElements !== 'undefined') {
    [
      I18nT$1
    ].forEach(cmp => {
      if (!customElements.get(cmp.is)) {
        customElements.define(cmp.is, cmp, opts);
      }
    });
  }
};

export { I18nT$1 as I18nT, defineCustomElements };
